<?php
/**
 * An interface for a package fragmenter
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2015 Hub Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
interface Fragmentable extends FrameworkInterface {
	/**
	 * This method does "implode" the given package data array into one long
	 * string, splits it into small chunks, adds a serial number and checksum
	 * to all chunks and prepends a final hashsum chunk. It will return the
	 * final hash for faster processing of packages.
	 *
	 * @param	$packageData		Raw package data array
	 * @param	$helperInstance		An instance of a ConnectionHelper class
	 * @return	$finalHash			Final hash for faster processing
	 */
	function fragmentPackageArray (array $packageData, ConnectionHelper $helperInstance);

	/**
	 * This method gets the next chunk from the internal FIFO which should be
	 * sent to the given recipient. It will return an associative array where
	 * the key is the chunk hash and value the raw chunk data.
	 *
	 * @param	$finalHash		Final hash for faster lookup
	 * @return	$rawDataChunk	Raw package data chunk
	 */
	function getNextRawDataChunk ($finalHash);
}

// [EOF]
?>
