<?php
/**
 * A default peer pool class
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2015 Hub Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
interface PoolablePeer extends Poolable, SocketTag {
	/**
	 * Adds a socket resource to the peer pool
	 *
	 * @param	$socketResource		A valid (must be!) socket resource
	 * @param	$connectionType		Type of connection, can only be 'incoming', 'outgoing' or 'server'
	 * @return	void
	 * @throws	InvalidSocketException	If the given resource is invalid or errorous
	 * @throws	InvalidConnectionTypeException	If the provided connection type is not valid
	 */
	function addPeer ($socketResource, $connectionType);

	/**
	 * Getter for array of all socket resource arrays
	 *
	 * @return	$sockets	An array with all socket arrays
	 */
	function getAllSockets ();

	/**
	 * Getter for array of all socket resources
	 *
	 * @return	$sockets	An array with all sockets
	 */
	function getAllSingleSockets ();

	/**
	 * "Getter" for all sockets of specified type
	 *
	 * @param	$connectionType		Type of connection, can only be 'incoming', 'outgoing' or 'server'
	 * @return	$sockets			An array with sockets of given type
	 * @throws	InvalidConnectionTypeException	If the provided connection type is not valid
	 */
	function getSocketsByConnectionType ($connectionType);
}

// [EOF]
?>
