<?php
/**
 * A factory class for protocol handlers
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2015 Hub Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 * @todo		Unfinished stuff
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class ProtocolHandlerFactory extends ObjectFactory {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Returns a protocol handler based on given protocol type. This is mostly 'tcp'.
	 *
	 * @param	$protocolType		Protocol type to create/return a protocol handler instance for
	 * @return	$handlerInstance	A protocol handler instance
	 */
	private static function createProtocolHandlerByType ($protocolType) {
		// Do we have an instance in the registry?
		if (Registry::getRegistry()->instanceExists($protocolType . '_protocol_handler')) {
			// Then use this instance
			$handlerInstance = Registry::getRegistry()->getInstance($protocolType . '_protocol_handler');
		} else {
			// Now prepare the tags instance
			$handlerInstance = self::createObjectByConfiguredName($protocolType . '_protocol_handler_class');

			// Set the instance in registry for further use
			Registry::getRegistry()->addInstance($protocolType . '_protocol_handler', $handlerInstance);
		}

		// Return the instance
		return $handlerInstance;
	}

	/**
	 * Returns a singleton protocol handler instance from given HubHelper
	 * instance. If an instance is found in registry, it will be returned, else
	 * a new instance is created and stored in the same registry entry.
	 *
	 * @param	$helperInstance		An instance of a HubHelper class
	 * @return	$handlerInstance	A protocol handler instance
	 */
	public static final function createProtocolHandlerFromRecipientHelper (HubHelper $helperInstance) {
		// Get the protocol type from given helper instance
		$protocolType = $helperInstance->determineProtocolType();

		// Call super factory method
		return self::createProtocolHandlerByType($protocolType);
	}

	/**
	 * Creates an instance of a protocol handler from given (raw) package data
	 *
	 * @param	$packageData		An array with raw package data
	 * @return	$handlerInstance	A protocol handler instance
	 */
	public static final function createProtocolHandlerFromPackageData (array $packageData) {
		// "Discover" the protocol type
		$protocolType = ProtocolDiscovery::discoverProtocolByPackageData($packageData);

		// Call super factory method
		return self::createProtocolHandlerByType($protocolType);
	}
}

// [EOF]
?>
