<?php
/**
 * A ChunkAssembler node-task
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2015 Hub Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
class NodeChunkAssemblerTask extends BaseTask implements Taskable, Visitable {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this class
	 *
	 * @return	$taskInstance	An instance of a Visitable class
	 */
	public final static function createNodeChunkAssemblerTask () {
		// Get new instance
		$taskInstance = new NodeChunkAssemblerTask();

		// Get a chunk handler instance
		$handlerInstance = ChunkHandlerFactory::createChunkHandlerInstance();

		// And add it to this task instance
		$taskInstance->setHandlerInstance($handlerInstance);

		// Return the prepared instance
		return $taskInstance;
	}

	/**
	 * Accepts the visitor to process the visitor
	 *
	 * @param	$visitorInstance	An instance of a Visitor class
	 * @return	void
	 * @todo	Also visit some sub-objects?
	 */
	public function accept (Visitor $visitorInstance) {
		// Visit this task
		$visitorInstance->visitTask($this);
	}

	/**
	 * Executes the task
	 *
	 * @return	void
	 */
	public function executeTask () {
		// "Cache" handler instance
		$handlerInstance = $this->getHandlerInstance();

		// Are there chunks to handle or a final array to assemble?
		if ($handlerInstance->ifUnassembledChunksAvailable()) {
			/*
			 * Then do the final steps:
			 *
			 * 1) Sort the final array with ksort(). This will bring the "hash
			 *    chunk" up to the last array index and the EOP chunk to the
			 *    pre-last array index
			 * 2) Assemble all chunks except two last (see above step)
			 * 3) While so, do the final check on all hashes
			 * 4) If the package is assembled back together, hash it again for
			 *    the very final verification.
			 */
			$handlerInstance->assembleChunksFromFinalArray();
		} elseif ($handlerInstance->ifUnhandledChunksWithFinalAvailable()) {
			/*
			 * Then handle them (not all!). This should push all chunks into a
			 * 'final array' for last verification.
			 */
			$handlerInstance->handleAvailableChunksWithFinal();
		} elseif ($handlerInstance->ifRawPackageDataIsAvailable()) {
			/*
			 * The final raw package data is back together again. So feed it
			 * into the next stack for further decoding/processing
			 */
			$handlerInstance->handledAssembledRawPackageData();
		}
	}

	/**
	 * Shuts down the task
	 *
	 * @return	void
	 * @todo	0% done
	 */
	public function doShutdown () {
		self::createDebugInstance(__CLASS__)->debugOutput('TASK[' . __METHOD__ . ':' . __LINE__ . ']: Shutting down...');
	}
}

// [EOF]
?>
